@@ -90,11 +90,12 @@ class BasicTests(BaseTestCase):
 
     def test_invalidate_by_boolean(self):
         count = Post.objects.cache().filter(visible=True).count()
+        with self.assertNumQueries(0):
+            Post.objects.cache().filter(visible=True).count()
 
         post = Post.objects.get(pk=1, visible=True)
         post.visible = False
         post.save()
-
         with self.assertNumQueries(1):
             new_count = Post.objects.cache().filter(visible=True).count()
             self.assertEqual(new_count, count - 1)
@@ -294,7 +295,8 @@ class DecoratorTests(BaseTestCase):
         view(factory.get('/hi'))
 
 
-from datetime import date, datetime, time
+from datetime import date, time
+from django.utils import timezone
 
 class WeirdTests(BaseTestCase):
     def _template(self, field, value):
@@ -313,7 +315,7 @@ class WeirdTests(BaseTestCase):
 
     def test_datetime(self):
         # NOTE: some databases (mysql) don't store microseconds
-        self._template('datetime_field', datetime.now().replace(microsecond=0))
+        self._template('datetime_field', timezone.now().replace(microsecond=0))
 
     def test_time(self):
         self._template('time_field', time(10, 30))
@@ -783,6 +785,15 @@ class AggregationTests(BaseTestCase):
         with self.assertNumQueries(1):
             qs.aggregate(posts_count=Count('posts'))
 
+    def test_aggregate_granular(self):
+        cat1, cat2 = Category.objects.all()[:2]
+        qs = Category.objects.cache().filter(id=cat1.id)
+        qs.aggregate(posts_count=Count('posts'))
+        # Test invalidation
+        Post.objects.create(title='New One', category=cat2)
+        with self.assertNumQueries(0):
+            qs.aggregate(posts_count=Count('posts'))
+
 
 class M2MTests(BaseTestCase):
     brand_cls = Brand
@@ -829,11 +840,10 @@ class M2MTests(BaseTestCase):
     def test_granular_through_on_clear(self):
         through_qs = self.brand_cls.labels.through.objects.cache() \
                                                   .filter(brand=self.bs, label=self.slow)
-        self._template(
-            lambda: through_qs.get(),
-            lambda: self.bf.labels.clear(),
-            should_invalidate=False
-        )
+        through_qs.get()
+        self.bf.labels.clear()
+        with self.assertNumQueries(0):
+            through_qs.get()
 
     def test_granular_target_on_clear(self):
         self._template(
